#include "global.h"

void arp_discovery(char *main_part, int first_host, int last_host)
{
  pcap_t *p_arp;
  fd_set rfd;
  struct bpf_program filter;
  struct pcap_pkthdr hdr;
  struct ethhdr *ethh;
  struct arphdr *arph;
  struct ifreq ifr;
  struct sockaddr_ll sa;
  struct timeval timeout;
  elem_t *new;
  unsigned char sip[50], packet[sizeof(struct ethhdr)+sizeof(struct arphdr)];
  unsigned char own_hw[ETH_ALEN], zero_hw[ETH_ALEN];
  unsigned char bcast_hw[ETH_ALEN] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
  unsigned char *pkt;
  char addr[50], filter_app[50], errbuf[256];
  long own_ip;
  int sock, pfd, i, net, mask;

  
  if((sock=socket(PF_PACKET, SOCK_RAW, 0))<0)
    { perror("socket: ");
      exit(1); }
  
  if(dev==NULL)
    if((dev=pcap_lookupdev(NULL))==NULL)
      { fprintf(stderr, "Couldn't find any interface!\n");
        exit(1); }
  
  printf("[X] Using %s\n", dev);  
  printf("[X] Discovery range: %s.%d - %s.%d\n", main_part, first_host, main_part, last_host);
  
  
  /*
   * here we can, of course, use fake hw address in arp request, 
   * but it'd be too visible that someting is wrong
   * so we're sending them, as "really we"
   *
   *
   * but if you really wanna do it, you can
   */


  memset(zero_hw, 0, 6);
  memset(&sa, 0, sizeof(struct sockaddr_ll));

  strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
  if(ioctl(sock, SIOCGIFINDEX, &ifr)<0)
    { perror("ioctl SIOCGIFINDEX");
      exit(1); }
                    
  sa.sll_ifindex=ifr.ifr_ifindex;
                          
  if (!fake_ok)
    {
      if(ioctl(sock, SIOCGIFHWADDR, &ifr)<0)
        { perror("ioctl SIOCGIFHWADDR: "); 
          exit(1); }
          
      memcpy(own_hw, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
   
      if(ioctl(sock, SIOCGIFADDR, &ifr)<0)
        { perror("ioctl SIOCGIFADDR: ");
          exit(1); } 
      memcpy((void *)&own_ip, ifr.ifr_addr.sa_data+2, 4);    
    }
  else
    {
      sprintf(own_hw, "%x%x%x%x%x%x", rand()%5334223, rand()%5334223, rand()%5334223, 
                                      rand()%5334223, rand()%5334223, rand()%5334223);

      own_ip = rand()%5334223;
      }
  

  sa.sll_family=AF_INET;
  sa.sll_protocol=htons(ETH_P_ARP);
  
      
  ethh=(struct ethhdr *)packet;
  memcpy(ethh->h_dest, bcast_hw, ETH_ALEN);
  memcpy(ethh->h_source, own_hw, ETH_ALEN);
  ethh->h_proto=htons(ETH_P_ARP);
  

  arph=(struct arphdr *)(packet+14);
  arph->ar_hrd=htons(ARPHRD_ETHER);
  arph->ar_pro=htons(ETH_P_IP);
  arph->ar_hln=6;
  arph->ar_pln=4;
  arph->ar_op=htons(ARPOP_REQUEST);
  
  memcpy(arph->ar_sha, own_hw, ETH_ALEN);
  memcpy(arph->ar_sip, &own_ip, 4);
  memset(arph->ar_tha, 0, ETH_ALEN);
        
  printf("[*] Sending requests...");
  
  for(i=first_host;i<last_host;i++)
    {    
        snprintf(addr, sizeof(addr), "%s.%d", main_part, i);     
        inet_pton(PF_INET, addr, arph->ar_tip);
         
        if(sendto(sock, packet, sizeof(packet), 0, (struct sockaddr *)&sa, sizeof(sa))<0)
          perror("sendto() ");
    }
  
  close(sock);
  printf("\n");  
  
  /*
   * now we're catching reponses
   */
  
   
  if((p_arp = pcap_open_live(dev, 40, 1, 1, errbuf))==NULL)
    {
      fprintf(stderr, "Cannot open device: %s\n", errbuf);
      exit(1);
    }

  snprintf(filter_app, sizeof(filter_app), "arp[7]==2");

  if(pcap_lookupnet(dev, &net, &mask, errbuf)<0)
    { fprintf(stderr, "%s\n", errbuf);
      exit(1); }

  if(pcap_compile(p_arp, &filter, filter_app, 0, net)<0)
    { fprintf(stderr, "%s\n", pcap_geterr(p_arp));
      exit(1); }
  
  if(pcap_setfilter(p_arp, &filter)<0)
    { fprintf(stderr, "Couldn't set filter\n");
      exit(1); }

  printf("[*] Start catching packets\n");


  pfd=pcap_fileno(p_arp);
  timeout.tv_sec = (last_host - first_host)*0.6;
  FD_ZERO(&rfd);
  FD_SET(pfd, &rfd);
  
  while(select(pfd+1, &rfd, NULL, NULL, &timeout)>0)
    {
      pkt=(char *)pcap_next(p_arp, &hdr);
      ethh = (struct ethhdr *)pkt;
      arph = (struct arphdr *)(pkt + sizeof(struct ethhdr));


      if ((ntohs(arph->ar_op) == ARPOP_REPLY) &&
         ((arph->ar_sip[4] >= first_host) || (arph->ar_sip[4] <= last_host)))
          {
            sprintf(sip, "%d.%d.%d.%d", arph->ar_sip[0], arph->ar_sip[1],
 	            arph->ar_sip[2], arph->ar_sip[3]);
 
            printf("\tResponse from %s\n", sip);
            new = list_elem(arph->ar_sha, sip);
            list_add(&addr_list, new);
          }
      
      FD_SET(pfd, &rfd);
    }
  
  pcap_close(p_arp);
  
  list_sync(addr_list);
}
